/*
 * 	debug.h
 * 	
 * 	Josef Kufner <jkufner@seznam.cz>
 *
 * 	2006-04-29
 * 		* Zobrazovani nazvu metod a funkci
 *
 * 	2002-09-06
 * 		* Makra pro zobrazovani ladicich vypisu
 *
 */

/*
 *  Hodnota makra DEBUG:
 *     0 ... Zadne vypisy
 *     1 ... Malo vypisu - jen kriticke chyby
 *     :
 *     :
 *    10 ... Uplne vsechny vypisy
 *
 *
 * DEBUG		... uroven vypisu
 * FORCE_DEBUG		... pokud neni -1, tak prebiji DEBUG
 * CLASS 		... nazev tridy
 * NOCLASS		... trida neni, pis jen nazev funkce
 * DEBUG_PREFIX_STR 	... prefix textu vypisu
 * DEBUG_PREFIX_ARG 	... parametry v prefixu, oddelene carkou
 * 
 */
#include <stdio.h>

#if defined(FORCE_DEBUG) && FORCE_DEBUG != -1
#	define DEBUG_LEVEL FORCE_DEBUG
#else
#	ifndef DEBUG_LEVEL
#		define DEBUG_LEVEL DEBUG
#	endif
#endif

#ifdef DBG_MSG
#	undef DBG_MSG
#endif

#ifndef __func__
#	define __func__   __FUNCTION__
#endif

#if DEBUG_LEVEL > 0
#ifndef DEBUG_PREFIX_STR
#define DEBUG_PREFIX_STR "%s"
#define DEBUG_PREFIX_ARG ""
#endif
#ifndef DEBUG_PREFIX_ARG
#define DEBUG_PREFIX_ARG ""
#else
#define DEBUG_PREFIX_ARG_X , DEBUG_PREFIX_ARG
#endif

#ifdef CLASS
#define DBG_MSG(level, msg, arg...)					\
{									\
	if (level <= DEBUG_LEVEL)					\
		fprintf(stderr, "%s:%d: %s::%s: " DEBUG_PREFIX_STR msg "\n", \
			__FILE__, __LINE__, 				\
			CLASS, __func__					\
			DEBUG_PREFIX_ARG_X,				\
			##arg);						\
}
#elif defined(NOCLASS)
#define DBG_MSG(level, msg, arg...)					\
{									\
	if (level <= DEBUG_LEVEL)					\
		fprintf(stderr, "%s:%d: %s: " DEBUG_PREFIX_STR msg "\n", \
			__FILE__, __LINE__, __func__ 			\
			DEBUG_PREFIX_ARG_X,				\
			##arg);						\
}
#else
#define DBG_MSG(level, msg, arg...)					\
{									\
	if (level <= DEBUG_LEVEL)					\
		fprintf(stderr, "%s:%d: " DEBUG_PREFIX_STR msg "\n", 	\
			__FILE__, __LINE__				\
			DEBUG_PREFIX_ARG_X,				\
			##arg);						\
}
#endif
#else
#	define DBG_MSG(level, msg, arg...)
#endif

