/*
 * fifo-test.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "fifo.h"
#include "ref.h"

#define NOCLASS
#undef FORCE_DEBUG
#define DEBUG 5
#define DEBUG_PREFIX_STR	"%d: "
#define DEBUG_PREFIX_ARG	thread_id
#include "debug.h"

#define THREADS 50

class Data : public Ref {
	public:
		Data(){
			junk[10] = 'a';
			junk[1024] = 'b';
			junk[2048] = 'b';
			junk[4000] = 'c';
			timestamp = time(NULL);
		}
		int i;
		char junk[4096];
		int timestamp;
};

typedef Fifo<Data *> MyFifo;

MyFifo *fifo = NULL;

void *read_fifo_thread(void *arg) {
	Data *d;
	int sum = 0;
	int thread_id = (int)arg;

	DBG_MSG(2, "Hello.");
	
	fifo->target_ref();
	while (fifo->pop(&d)) {
		DBG_MSG(3, "rcv %d", d->i);
		sum += d->i;
		usleep(rand() % 100 + 10000);
		d->unref();
	}
	fifo->target_unref();

	DBG_MSG(2, "Result: %d", sum);
	return (void*) new int(sum);
}

int main (void) {
	int i, res = 0, launched = 0, total = 0, *p = NULL;
	pthread_t thread[THREADS];
	int thread_id = 0;

	fprintf(stderr, "Fifo selftest\n");
	//srand(time(NULL));

	DBG_MSG(1, "Creating fifo...");

	fifo = new MyFifo("debug fifo");
	
	DBG_MSG(1, "Creating threads...");
	for (i = 0; i < THREADS; i++) {
		if(pthread_create(&(thread[i]), NULL, read_fifo_thread, (void *)(i + 1))) {
			perror("pthread_create");
			break;
		} else {
			launched++;
		}
	}
	DBG_MSG(1, "%d threads.", launched);
	
	DBG_MSG(1, "Sending data...");
	for (i = 0; i < 300; i++) {
		Data *d = new Data();
		d->i = i;
		res += d->i;
		DBG_MSG(3, "snd %d\n", d->i);
		fifo->push(d);
		d->unref();
		usleep(rand() % 100 + 1);
	}
	fifo->source_unref();
	DBG_MSG(1, "Result should be %d", res);
	
	DBG_MSG(1, "Joining threads...");
	for (i = 0; i < launched; i++) {
		pthread_join(thread[i], (void**)&p);
		total += *p;
	}
	DBG_MSG(1, "Total: %d, should be %d... %s", total, res * launched,
			total == res * launched ? "OK":"ERROR");

	DBG_MSG(1, "Removing fifo...");
	delete fifo;

	DBG_MSG(1, "End.");
	return total == res * launched ? EXIT_SUCCESS : EXIT_FAILURE;

}

