/*
 * game_beast.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#include "game_beast.h"
#include "game_item.h"

#define DEBUG 0
#include "debug.h"

GBeast::GBeast(void) {
	direction = RIGHT;
	old_direction = RIGHT;
	speed = 0;
	waiting = 0;
	score = 0;
	add_property(GOP_SCORE, &score);
	add_property(GOP_SPEED, &speed);
}

bool GBeast::calculate_step(int &new_x, int &new_y, int &new_z) {
	int ret = 0;

	new_x = pos_x;
	new_y = pos_y;
	new_z = pos_z;

	if (!waiting) {
		switch (direction) {
			case UP:
				if (pos_y > 0) {
					new_y--;
					ret = 1;
				}
				break;
			case DOWN:
				if (pos_y < parent->get_height() - 1) {
					new_y++;
					ret = 1;
				}
				break;
			case LEFT:
				if (pos_x > 0) {
					new_x--;
					ret = 1;
				}
				break;
			case RIGHT:
				if (pos_x < parent->get_width() - 1) {
					new_x++;
					ret = 1;
				}
				break;
			default:
				break;
		}
		old_direction = direction;

		/* position accessibility test */
		local_obj.clear();
		local_obj = parent->get_child_at(new_x, new_y);
		for (GObjectList::iterator i = local_obj.begin(); i != local_obj.end(); i++) {
			if (!(*i)->is_accessible()) {
				new_x = pos_x;
				new_y = pos_y;
				new_z = pos_z;
			}
		}	
		
		DBG_MSG(5, "GBeast::calculate_step: new step of \"%s\" is [ %d, %d, %d ] -> [ %d, %d, %d ], speed = %d",
				name, pos_x, pos_y, pos_z, new_x, new_y, new_z, speed);
		waiting = speed;
		return true;
	} else {
		waiting--;
		DBG_MSG(5, "GBeast::calculate_step: %d step remaining, speed = %d.", waiting, speed);
		return false;
	}
}

void GBeast::step(void) {
	int new_x, new_y, new_z;
	
	if (calculate_step(new_x, new_y, new_z)) {
		pos_x = new_x;
		pos_y = new_y;
		pos_z = new_z;
		after_step();
	}
	return;
}

void GKrtek::after_step(void) {
	/*
	GObjectList here = parent->get_child_at(pos_x, pos_y);
	for (GObjectList::iterator i = here.begin(); i != here.end(); i++) {
		DBG_MSG(6, "GKrtek::after_step: here is %s in layer %d", (*i)->get_name(), (*i)->get_pos_z());
	}
	*/
	for (GObjectList::iterator i = local_obj.begin(); i != local_obj.end(); i++) {
		if ((*i)->get_goid() == GOID_TRAVA) {
			speed = ((*i)->get_property(GOP_DAMAGE)) > 0 ? 2 : 4;
			waiting = speed;
			(*i)->set_property(GOP_DAMAGE, 1000);
		}
	}	
	return;
}

void GZahradnik::after_step(void) {
	for (GObjectList::iterator i = local_obj.begin(); i != local_obj.end(); i++) {
		if ((*i)->get_goid() == GOID_TRAVA) {
			speed = ((*i)->get_property(GOP_DAMAGE) > 0) ? 8 : 3;
			waiting = speed;
			(*i)->set_property(GOP_DAMAGE, 0);
		} else if ((*i)->get_goid() == GOID_KRTEK) {
			score += 20;
		}
	}
	if (kyticka <= 0) {
		GObject* kytka = new GKyticka();
		kytka->set_parent(parent);
		kytka->set_position(pos_x, pos_y, pos_z);
		parent->add_child(kytka);
		kyticka = 7;
	} else {
		kyticka--;
	}
	return;
}


