/*
 * game_glmodel.h
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#ifndef JK_GAME_GLMODEL_H
#define JK_GAME_GLMODEL_H

#include <stdio.h>
#include <vector>
#include <list>
#include <string>
#include <map>

#include "ref.h"

class GGLModel : public Ref {
	public:
		GGLModel(void);
		~GGLModel(void);

		/* Load model */
		bool load(char *filename);

		/* Draw model */
		bool draw(void);

		/* Resize model (optional) */
		void force_size(float size, bool discard_offset) {
			size_forced = true;
			size_scale = size;
			size_nooffset = discard_offset;
		}
		
		/* Load materials -- called by load() */
		bool load_materials(char *basefilename, char *filename);

	private:
		typedef struct {
			float r, g, b;
		} COLOR;
		
		typedef struct {
			COLOR ambient, diffuse, specular;
			float alpha, shininess;

			bool have_texture;
			std::string texture;
			bool texture_loaded;
			GLuint texture_id;
		} MATERIAL;

		typedef std::map<std::string,MATERIAL> MATERIAL_MAP;

		typedef struct {
			float x, y, z;
		} VERTEX;
		
		typedef struct {
			int g;	// geometric vertex
			int t;	// texture vertex
			int n;	// normal vertex
			MATERIAL *material;
		} FACE_VERTEX;
		
		typedef std::vector<VERTEX>	VERTEX_ARRAY;
		typedef std::list<FACE_VERTEX>	FACE;
		typedef std::list<FACE>		FACE_ARRAY;

		MATERIAL_MAP material;
		VERTEX_ARRAY geom_vertex;
		VERTEX_ARRAY norm_vertex;
		VERTEX_ARRAY texture_vertex;
		FACE_ARRAY face;

		void skip_to_eol(FILE *f);
		bool skip_space(FILE *f);
		char *make_filename(char *dir_file, char *file);
		
		bool size_forced, size_nooffset;
		float size_scale;
		float size_max[3], size_min[3];

};

#endif

