/*
 * game_glmodel.test.c
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include "glut_master.h"
#include "game_glmodel.h"

#define DEBUG 5
#define NOCLASS
#include "debug.h"


class ModelView : GlutWindow {
	public:
		ModelView(GlutMaster *glut_master, GGLModel *model) {
			angle_step = 360.0 / 128.0;
			angle_x = angle_step * 8.0;
			angle_y = 0;
			angle_z = 0;
			distance = 1.7;
			distance_step = -0.1;
			rotate = false;
			rotation_start = true;
			rotation_speed = angle_step * 3.0;
			light = true;
			
			this->model = model;
			this->glut_master = glut_master;
			glut_master->CallGlutCreateWindow("game_glmodel self test", this);
			glEnable(GL_DITHER);
			glEnable(GL_DEPTH_TEST);
			glEnable(GL_TEXTURE_2D);
	
			if (rotate)
				rotation_enable();
		}
		
	private:
		GlutMaster *glut_master;
		GGLModel *model;
		int win_width, win_height;
		float angle_x, angle_y, angle_z, angle_step;
		float distance, distance_step;
		bool rotate, rotation_start;
		float rotation_speed;
		bool light;

		void CallBackReshapeFunc(int width, int height);
		void CallBackIdleFunc (void);
		void CallBackVisibleFunc (int vis);
		void CallBackDisplayFunc(void);
		void CallBackKeyboardFunc(unsigned char key, int x, int y);
		void CallBackSpecialFunc(int key, int x, int y);

		void rotation_enable(bool toggle = false) {
			rotate = toggle ? !rotate : true;
			if (rotate) {
				rotation_start = true;
				glut_master->EnableIdleFunction();
				glut_master->SetIdleToCurrentWindow();
			} else {
				glut_master->DisableIdleFunction();
			}
		}

		void rotation_disable(void) {
			rotate = false;
			glut_master->DisableIdleFunction();
		}
};

void ModelView::CallBackReshapeFunc(int width, int height) {
	DBG_MSG(4, "");
	glViewport(0, 0, width, height);
	win_width = width;
	win_height = height;
	glutPostRedisplay();
}

void ModelView::CallBackIdleFunc (void) {
	static float last_pos = 0;
	if (rotate) {
		float pos = (float)((int)(glutGet(GLUT_ELAPSED_TIME) % (360 * 400))) / 400.0;
		if (rotation_start) {
			rotation_start = false;
			last_pos = pos;
		} else {
			angle_y += rotation_speed * (pos - last_pos);
			last_pos = pos;
			usleep(100);	// limit framerate
		}
	}
	glutPostRedisplay();
}                                                                                                                
                                                                                                                 
void ModelView::CallBackVisibleFunc (int vis) {
	if (vis == GLUT_VISIBLE && rotate) {
		glut_master->EnableIdleFunction();
		glut_master->SetIdleToCurrentWindow();
	} else {
		glut_master->DisableIdleFunction();
	}
}                                                                                                                


void ModelView::CallBackDisplayFunc(void) {
	DBG_MSG(6, "");

	/* reset */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(63.4, ((float)win_width)/((float)win_height), 0.1, 10000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* nastaveni svetla */
	GLfloat light_position[] = { 10.0f, 10.0f, 10.0f, 0.0f };
	GLfloat light_color[] = { 1.0, 1.0, 1.0 };
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_color);
	glEnable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);

	/* uprava pohledu */
	glTranslatef(0.0, 0.0, -distance);
	glRotatef(angle_x, 1.0, 0.0, 0.0);
	glRotatef(angle_y, 0.0, 1.0, 0.0);
	glRotatef(angle_z, 0.0, 0.0, 1.0);

	/* Vymazani */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* Zakladni kostka 1 x 1 x 1, stred v pocatku */
	glPointSize(7);
	glBegin(GL_POINTS);
	glColor3f(1.0, 1.0, 1.0); glVertex3f(-0.5, -0.5, -0.5); // white 0
	glColor3f(1.0, 0.0, 0.0); glVertex3f( 0.5, -0.5, -0.5); // red   x
	glColor3f(0.0, 1.0, 0.0); glVertex3f(-0.5,  0.5, -0.5); // green y
	glColor3f(0.0, 0.0, 1.0); glVertex3f(-0.5, -0.5,  0.5); // blue  z
	glEnd();
	glColor3f(0.5, 0.5, 0.5);
	glutWireCube(1.0);
	glBegin(GL_LINES);
	glVertex3f(+0.5, -0.5, +0.5); glVertex3f(-0.5, -0.5, -0.5);
	glVertex3f(-0.5, -0.5, +0.5); glVertex3f(+0.5, -0.5, -0.5);
	glEnd();

	/* povoleni svetla pro kresleni modelu */
	if (light)
		glEnable(GL_LIGHTING);

	/* vykresleni modelu, pocatek ve stredu podstavy jednotkove kostky */
	if (model) {
		glTranslatef(0.0, -0.5, 0.0);
		model->draw();
	}

	/* Ok, to je vse */
	glFlush();
	glutSwapBuffers();
}

void ModelView::CallBackKeyboardFunc(unsigned char key, int x, int y) {
	DBG_MSG(5, "key = %c (%d); x = %d; y = %d", key, key, x, y);
	switch (key) {
		case 'q':
		case 'Q':
		case 27:
			exit(0);
			break;

		case '+':
		case '=':
			distance += distance_step;
			break;
		case '-':
		case '_':
			distance -= distance_step;
			break;

		case ' ':
			rotation_enable(true);
			break;

		case 'l':
		case 'L':
			light = !light;
	}
	if (distance < 0.0)
		distance = 0.0;
	glutPostRedisplay();
}

void ModelView::CallBackSpecialFunc(int key, int x, int y) {
	DBG_MSG(5, "key = %d; x = %d; y = %d", key, x, y);
	switch (key) {
		case GLUT_KEY_LEFT:
			angle_y -= angle_step;
			break;
		case GLUT_KEY_RIGHT:
			angle_y += angle_step;
			break;
		case GLUT_KEY_UP:
			angle_x -= angle_step;
			break;
		case GLUT_KEY_DOWN:
			angle_x += angle_step;
			break;
	}
	DBG_MSG(4, "angle_x = %f; angle_y = %f", angle_x, angle_y);
	glutPostRedisplay();
}

int main (int argc, char *argv[]) {
	ModelView *model_view = NULL;
	GlutMaster *glut_master = NULL;
	GGLModel *model = NULL;

	DBG_MSG(1, "Start...");

	DBG_MSG(1, "GL init...");
	glut_master = new GlutMaster(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glEnable(GL_DITHER);

	DBG_MSG(1, "Model init...");
	if (argc == 2) {
		model = new GGLModel();
		if (!model->load(argv[1])) {
			DBG_MSG(1, "Model loading Failed.");
			return 1;
		}
		model->force_size(1.0, true);
	}

	DBG_MSG(1, "ModelView init...");
	model_view = new ModelView(glut_master, model);
	
	DBG_MSG(1, "GlutMaster main loop...");
	glut_master->CallGlutMainLoop();
	
	DBG_MSG(1, "End.");
}


