/*
 * game_object.h
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#ifndef JK_GAME_OBJECT_H
#define JK_GAME_OBJECT_H

#include <list>
#include <map>

/*****************************************************************************
 * Symbol na vykreslovani objektu
 */
class GSymbol {
	public:
		GSymbol(int goid = 0, char symbol = 0, int color = 0, int x = 0, int y = 0, int z = 1, int dir = 0) {
			this->goid = goid;
			this->symbol = symbol;
			this->color = color;
			this->x = x;
			this->y = y;
			this->z = z;
			this->direction = dir;
		};
		
		/*
		GSymbol(const GSymbol &sym) {
			this->symbol = sym.symbol;
			this->color = sym.color;
			this->x = sym.x;
			this->y = sym.y;
			this->z = sym.z;
		};
		
		operator bool(){
			return this->symbol != 0;
		}
		*/
			
		int goid;
		int x, y, z;
		int color;
		char symbol;
		int direction;	// degrees
};

/*****************************************************************************
 * GObject -- zakladni trida
 *
 * Prostor:
 * 	- 3 rozmery: 2D plocha + vrstva
 * 	- vrstvy:
 * 		- zem
 * 		- povrch
 *
 *
 *
 * Souradnicovy system:
 *
 *              [0,0,0]                          x ... left -> right
 *	              +---------------------     y ... up -> down
 *                   /|                  x       z ... layer (0 is lowest)
 *	            / |
 *	           /  |
 * 	          /   |
 * 	       z /    |
 * 	        /     | y
 * 	              |
 * 		
 * 
 */
class GObject;

/*
class GObjectPointer {
	public:
		GObjectPointer(void) {
			target = (GObject *)0;
		}
	private:
		GObject *target;

		friend bool operator==(const GObject& a, const GObject& b) {
			return calc_object_key(a) == calc_object_key(b);
		};
		friend bool operator!=(const GObject& a, const GObject& b) {
			return calc_object_key(a) != calc_object_key(b);
		};
		friend bool operator<(const GObject& a, const GObject& b) {
			return calc_object_key(a) < calc_object_key(b);
		};
		friend bool operator>(const GObject& a, const GObject& b) {
			return calc_object_key(a) > calc_object_key(b);
		};
		friend bool operator<=(const GObject& a, const GObject& b) {
			return calc_object_key(a) <= calc_object_key(b);
		};
		friend bool operator>=(const GObject& a, const GObject& b) {
			return calc_object_key(a) >= calc_object_key(b);
		};
};
*/

typedef std::list<GObject *> GObjectList;
typedef std::multimap<int, GObject *> GObjectMap;
typedef int GOPType;
typedef std::map<const int, GOPType*> GOPList;

class GObject {
	public:
		/* identifikace objektu, nemusi byt unikatni */
		enum {
			GOID_UNDEFINED = 0,
			GOID_ZAHRADNIK,
			GOID_KRTEK,
			GOID_KYTICKA,
			GOID_TRAVA,
			GOID_VODA,
			GOID_MAX
		};

		/* jmena vsech moznych vlastnosti vseho */
		enum {
			GOP_POS_X, GOP_POS_Y, GOP_POS_Z,
			GOP_WIDTH, GOP_HEIGHT, GOP_DEPTH,
			GOP_SCORE, GOP_SPEED,
			GOP_DAMAGE
		};
		
		
		GObject(void);
		virtual ~GObject(void);

		virtual GSymbol* draw(void) = 0;
		virtual void step(void);
		virtual bool is_accessible(void) { return true; }
		void die (void) { death = true; };
		bool is_dead(void) { return death; };

		/* identifikace */
		const char *get_name(void) { return name; };
		int get_goid(void) { return goid; };

		/* nastavovani/ziskavani vlastnosti */
		virtual void add_property(const int prop_id, GOPType *var, bool readonly = false);
		virtual bool set_property(const int prop_id, GOPType val);
		virtual bool add_to_property(const int prop_id, GOPType val);
		virtual GOPType get_property(const int prop_id);

		/* poloha */
		void set_position(int pos_x, int pos_y, int pos_z);
		int get_pos_x(void) { return pos_x; };
		int get_pos_y(void) { return pos_y; };
		int get_pos_z(void) { return pos_z; };

		/* rozmery */
		int get_height(void) { return height; };
		int get_width(void) { return width; };
		int get_depth(void) { return depth; };

		/* Manipulace s rodicem
		 * pokud je parent == NULL, tak rodic proste nejni
		 */
		GObject *get_parent(void) { return parent; };
		void set_parent(GObject *parent) { this->parent = parent; };

		/* Manipulace s potomky
		 * 
		 * get_rewind nastavi ukazatel na zacatek seznamu
		 * get_next vraci dalsi a dalsi objekty, dokud nevrati NULL
		 * add_child prida potomka a nevrati nic
		 */
		void get_child_rewind(void) { get_child_iter = child.begin(); };
		GObject *get_child(void) {
			if (get_child_iter != child.end())
				return *(get_child_iter++);
			else
				return NULL;
		};
		void add_child(GObject *child) { this->child.push_back(child); };

		/* Zjistovani potomku podle pozice
		 * nutno zavolat update_get_child_at() pred zjistovanim, pokud
		 * od posledne mohla probehnout nejaka zmena nebo je to poprve
		 */
		void update_get_child_at(void);
		GObjectList get_child_at(int x, int y);

		/* vyber specialniho potomka */
		GObject *get_child_by_goid(int goid);
		
	protected:
		int goid;
		char *name;
		bool death;
		GOPList gop_list;
		GObject *parent;
		GObjectList child;
		GObjectList::iterator get_child_iter;
		GObjectMap child_map;
		
		int height, width, depth;
		int pos_x, pos_y, pos_z;
};

#endif
