/*
 * game_world.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include "game_world.h"
#include "game_object.h"
#include "game.h"
#include "ref.h"

#define DEBUG 0
#include "debug.h"

GameWorld::GameWorld(int width, int height) {
	GObject *obj = NULL;
	zahradnik = NULL;
	krtek = NULL;
	this->width = /*width*/ 74;
	this->height = /*height*/ 20;

	this->depth = 2;	/* 0: background
				 * 1: item + beast
				 */
	
	DBG_MSG(2, "GameWorld::GameWorld: Creating world: width = %d, height = %d", this->width, this->height);
	
	char mapa[20][75] = {
                  "..........................................................................",
                  "..........................................................................",
                  "..........................................................................",
                  "..........................................................................",
                  "....................................~~~~~~................................",
                  "...................................~~~~~~~~...............................",
                  ".....................................~~~~~~...............................",
                  "..........................~~~~~~~......~~~~~..............................",
                  "......................~~~~~~~~~~~~~.....~~~~~~~~..........................",
                  ".....................~~~~~~~~~~~~~~~~~.....~~~~~~~~~~~~~..................",
                  "...................~~~~~~~~~~~~~~~~~~~~~~..~~~~~~~~~~~~~~~~~~~~~~.........",
                  ".......................~~~~~~~~~~~~~~~~~~......~~~~~~~~~~~~~~~~~~~~~......",
                  ".......................~~~~~~~~~~~~~~~~~~~~~~~.~~~~~~~~~~~~~~~~~..........",
                  "...........................~~~~~~~~~............~~~~~~~~~~~...............",
                  "............................~~~~~~...............~~~~~~~~.................",
                  "..........................................................................",
                  "..........................................................................",
                  "..........................................................................",
                  "..........................................................................",
                  ".........................................................................."
	};


	// tvorba krtka
	krtek = new GKrtek();
	krtek->set_parent(this);
	krtek->set_position(width / 4, height / 4, 1);
	this->add_child(krtek);

	// tvorba zahradnika
	zahradnik = new GZahradnik();
	zahradnik->set_parent(this);
	zahradnik->set_position((width / 4) * 3, (height / 4) * 3, 1);
	this->add_child(zahradnik);

	// tvorba mapy -- bude nacitana za souboru
	/*
	for (int row = 0; row < height; row++) {
		for (int col = 0; col < width; col++) {
			mapa[row][col] = '.';
		}
	}
	*/
	
	// tvorba zeme
	for (int row = 0; row < height; row++) {
		for (int col = 0; col < width; col++) {
			switch (mapa[row][col]) {
				case '.':
					obj = new GTrava();
					break;
				case '~':
					obj = new GVoda();
					break;
				default:
					obj = NULL;
			}
			if (obj) {
				obj->set_parent(this);
				obj->set_position(col, row, 0);
				this->add_child(obj);
			}
		}
	}
	
	DBG_MSG(2, "GameWorld::GameWorld: Empty world completed");
	
}

GameWorld::~GameWorld(void) {
	DBG_MSG(2, "GameWorld::~GameWorld: Destroying world...");
}

void GameWorld::step(void) {
	GObjectList::iterator next;
	DBG_MSG(2, "GameWorld::step: One step for all...");
	
	update_get_child_at();
	
	GObjectList::iterator i = child.begin();
	while (i != child.end()) {
		if ((*i)->is_dead()) {
			DBG_MSG(3, "GameWorld::step: dead object detected - deleting...");
			next = i; next++;
			delete (*i);
			child.erase(i);
			i = next;
		} else {
			(*i)->step();
			i++;
		}
	}
	DBG_MSG(2, "GameWorld::step: Done");
}


void GameWorld::interpret_keys(int key[KEY_NUM]) {
	/* klavesy pro krtka */
	if (key[0]) {
		krtek->turn_left();
	}
	if (key[1]) {
		krtek->turn_right();
	}
	
	/* klavesy pro zahradnika */
	if (key[2]) {
		zahradnik->turn_left();
	}
	if (key[3]) {
		zahradnik->turn_right();
	}
}
