/*
 * lock.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */

#include <stdio.h>
#include <pthread.h>

#include "lock.h"

#define CLASS "Lock"
#define DEBUG 0
#include "debug.h"

Lock::Lock(void) {
	DBG_MSG(8, "");
	pthread_mutex_init(&lock, NULL);
	pthread_cond_init(&rd_cond, NULL);
	pthread_cond_init(&wr_cond, NULL);
	reading = 0;
	writing = 0;
	DBG_MSG(8, "Lock created");
}

Lock::~Lock(void) {
	DBG_MSG(8, "");
	pthread_mutex_destroy(&lock);
	pthread_cond_destroy(&rd_cond);
	pthread_cond_destroy(&wr_cond);
	DBG_MSG(8, "Lock destroyed");
}

void Lock::read_lock(void) {
	DBG_MSG(8, "waiting...");
	pthread_mutex_lock(&lock);

	// nikdo nesmi zapisovat
	while (writing) {
		DBG_MSG(8, "Waiting for writer...");
		pthread_cond_wait(&wr_cond, &lock);
	}

	reading++;
	DBG_MSG(8, "Read lock is ON.");
	
	pthread_mutex_unlock(&lock);
}

void Lock::read_unlock(void) {
	DBG_MSG(8, "waiting...");
	pthread_mutex_lock(&lock);

	if (reading > 0)
		reading--;

	DBG_MSG(8, "Read lock is %s (%d readers remains).",
			reading ? "ON" : "OFF", reading);
	
	pthread_mutex_unlock(&lock);
	pthread_cond_broadcast(&rd_cond);
}

void Lock::write_lock(void) {
	DBG_MSG(8, "waiting...");
	pthread_mutex_lock(&lock);

	// nikdo nesmi zapisovat
	while (writing) {
		DBG_MSG(8, "Waiting for writer...");
		pthread_cond_wait(&wr_cond, &lock);
	}
	
	writing++;

	// nikdo nesmi cist
	while (reading) {
		DBG_MSG(8, "Waiting for readers...");
		pthread_cond_wait(&rd_cond, &lock);
	}
	
	
	DBG_MSG(8, "Write lock is ON.");
	
	pthread_mutex_unlock(&lock);
}

void Lock::write_unlock(void) {
	DBG_MSG(8, "waiting...");
	pthread_mutex_lock(&lock);

	if (writing > 0)
		writing--;

	DBG_MSG(8, "Write lock is %s (%d writers remains).",
			writing ? "ON" : "OFF", writing);
	
	pthread_mutex_unlock(&lock);
	pthread_cond_broadcast(&wr_cond);
	pthread_cond_signal(&rd_cond);
}

#undef CLASS

