/*
 * lock-test.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <time.h>

#include "lock.h"

#define DEBUG 5
#include "debug.h"

#define THREADS 50

class SharedData : public Lock {
	public:
		SharedData(){
			id = 1;
			number = 0;
			total = 0;
		};
		
		int id;
		int number;
		int total;	
};

SharedData data;

void *read_thread(void *arg) {
	int last_id = 0, new_id = 1;
	int sum = 0;

	DBG_MSG(2, "Hello.");

	while(new_id > 0) {
		data.read_lock();
		new_id = data.id;
		usleep(100);
		if (new_id != last_id) {
			sum += data.number;
			last_id = new_id;
			DBG_MSG(2, "New id: %d, curr sum: %d", new_id, sum);
		}
		data.read_unlock();
		usleep(100);
	}

	data.write_lock();
	data.total += sum;
	data.write_unlock();
	
	DBG_MSG(2, "Result: %d", sum);
	return (void*) new int(sum);
}

int main (void) {
	int i, res = 0, launched = 0, total = 0, *p = NULL;
	pthread_t thread[THREADS];

	srand(time(NULL));
	
	DBG_MSG(1, "Creating threads...");
	for (i = 0; i < THREADS; i++) {
		if(pthread_create(&(thread[i]), NULL, read_thread, NULL)) {
			perror("pthread_create");
			break;
		} else {
			launched++;
		}
	}
	DBG_MSG(1, "%d threads.", launched);
	
	DBG_MSG(1, "Sending data...");
	for (i = 2; i < 300; i++) {
		data.write_lock();
		data.id = i;
		data.number = random() % 1000;
		res += data.number;
		data.write_unlock();
		DBG_MSG(3, "write: id = %d, number = %d\n", data.id, data.number);
		usleep(100000);
	}
	data.write_lock();
	data.id = -1;
	data.number = 0;
	data.write_unlock();

	DBG_MSG(1, "Result should be %d", res);
	
	DBG_MSG(1, "Joining threads...");
	for (i = 0; i < launched; i++) {
		pthread_join(thread[i], (void**)&p);
		total += *p;
	}
	DBG_MSG(1, "Total: %d, total rcv: %d, should be %d.", data.total, total, res * launched);

	if (data.total == total && total == res * launched) {
		DBG_MSG(1, "OK.");
	} else {
		DBG_MSG(1, "neee :(");
	}

	DBG_MSG(1, "End.");
	return total == res * launched ? EXIT_SUCCESS : EXIT_FAILURE;

}

