/*
 * main.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#include "game.h"
#include "sample.h"
#include "screen.h"
#include "timer.h"
#include "screen-curses.h"
#include "screen-gl.h"

#define NOCLASS
#define DEBUG 10
#include "debug.h"

int done = 0;

SampleFifo *input_fifo = NULL;
ScreenFifo *output_fifo = NULL;
Sample *sample = NULL;
ScreenCurses *screen_curses = NULL;
ScreenGL *screen_gl = NULL;
Game *game = NULL;
Timer *timer = NULL;


void *cleanup_thread(void *arg) {
	DBG_MSG(1, "");
	input_fifo->destroy();
	output_fifo->destroy();
	DBG_MSG(1, "Done.");
	return NULL;
}

void on_signal(int sig) {
	pthread_t ct;
	pthread_attr_t ctattr;
	
	DBG_MSG(1, "***** SIGINT *****");
	signal(SIGINT, SIG_DFL);
	pthread_attr_init(&ctattr);
	pthread_attr_setdetachstate(&ctattr, PTHREAD_CREATE_DETACHED);
	pthread_create(&ct, NULL, cleanup_thread, NULL);
	return;
}

void on_term_resize(int sig) {
	if (screen_curses)
		screen_curses->resize_notify();
}



int main (int argc, char *argv[]) {

	DBG_MSG(1, "Initialization...");
	input_fifo    = new SampleFifo("Input");
	output_fifo   = new ScreenFifo("Output");
	sample        = new Sample(input_fifo);
	screen_curses = new ScreenCurses(output_fifo);
	screen_gl     = new ScreenGL(&argc, argv, output_fifo);
	game          = new Game(input_fifo, output_fifo);
	timer         = new Timer(input_fifo);

	input_fifo->source_unref();	// sample ji ma privlastnenou
					// a tady neni potreba
	
	signal(SIGINT, on_signal);
	signal(SIGWINCH, on_term_resize);

	DBG_MSG(1, "Starting game...");
	game->main();

	DBG_MSG(1, "Destroying everything...");
	delete game;
	delete timer;
	delete sample;
	delete screen_curses;
	delete screen_gl;
	delete input_fifo;
	delete output_fifo;

	DBG_MSG(1, "Bye.");
	return EXIT_SUCCESS;
}

