/*
 * screen_window.c -- OpenGL screen window
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <GL/glut.h>
#include <math.h>
#include <signal.h>
#include <time.h>
#include <list>

#include "screen-gl.h"
#include "game_object.h"

#define CLASS "ScreenGL"
#define DEBUG 5
#include "debug.h"

//#define DEG(x)				((float)(x) / (2.0 * 3.141592))
#define RAD(x)					((float)(x) * 3.141592 / 180.0)

//static int screen_window_created = 0;

GlutMaster *ScreenGL::glut_master = 0;

/*****************************************************************************
 * Init
 */
ScreenGL::ScreenGL(int *argc, char *argv[], ScreenFifo *fifo) : Thread("ScreenGL", true) {
	DBG_MSG(5, "");

	actual_msg = NULL;
	
	// fifo
	this->fifo = fifo;
	fifo->target_ref();
	
	/* members init */
	title = "Krtek (%0.1f FPS)";
	world_width = 74, world_height = 20;
	world_border = 20000;
	
	border_dl = -1;
	
	angle_x = 0;
	angle_y = 0;
	angle_z = 60;
	pos_x = 0;
	pos_y = 0;
	pos_z = 5;
	move_step = 0.25;
	angle_step = 2;
	camera_mode = CAM_FREE;
	light_sun = true;
	light_camera = false;

	fps_old_time = 0;
	fps_update_rate = 3;
	
	mouse_move = 0;
	step = 0;
	sun_speed = 0.5;
	
	/* only one window */
	/*
	if (screen_window_created) {
		fprintf(stderr, "Critical Internal Error: ScreenWindow can be created only once !!!\n");
		abort();
	}
	screen_window_created = 1;
	*/

	/* GLUT Init */
	if (!glut_master) {
		glut_master = new GlutMaster(argc, argv);
		glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);
	}

	thread_launch();
}


/*****************************************************************************
 * CleanUp
 */
ScreenGL::~ScreenGL(void) {
	DBG_MSG(5, "");
	
}

/*****************************************************************************
 * exec
 */
void *ScreenGL::main (void) {
	DBG_MSG(5, "");
	
	glut_master->CallGlutCreateWindow(title, this);
	glut_master->EnableIdleFunction();
	glut_master->SetIdleToCurrentWindow();

	/* glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); */
	//glutReshapeWindow(640/2, 480/2);
	//glutSetCursor(GLUT_CURSOR_NONE);
	
	/* GL Init*/
	glDisable(GL_BLEND);
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_POINT_SMOOTH);
	glDisable(GL_LINE_SMOOTH);
	glDisable(GL_POLYGON_SMOOTH);
	glDisable(GL_LINE_STIPPLE);
	glDisable(GL_POLYGON_STIPPLE);
	glEnable(GL_DITHER);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);

	/** MAIN DISPLAY LIST **/
	/* create object */
	DBG_MSG(4, "Creating DisplayList...");
	border_dl = glGenLists(1);
	glNewList(border_dl, GL_COMPILE);

	/* konvicka */
	/*
	gen_material(0.6, 0.6, 1.0);
	glPushMatrix();
	glTranslatef(10, 1.8, 10);
	glutSolidTeapot(2);
	glPopMatrix();
	*/
	
	
	/*
	glPointSize(5);
	glBegin(GL_POINTS);
	gen_material(1, 1, 1);
	glVertex3f(0.0, 0.0, 0.0);
	gen_material(1, 0, 0);
	glVertex3f(10.0, 0.0, 0.0);
	gen_material(0, 1, 0);
	glVertex3f(0.0, 10.0, 0.0);
	gen_material(0, 0, 1);
	glVertex3f(0.0, 0.0, 10.0);
	glEnd();
	*/

	/* dark strip around
	 *    a  b    c  d
	 *  1 +  +    +  +
	 *       |    |
	 *  2 +--+----+--+
	 *       |    |
	 *  3 +--0----+--+
	 *       |    |
	 *  4 +  +    +  +
	 */
	gen_material(0.1, 0.3, 0.0);
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0, 0, 1);
	/* b 3 */ glVertex3f(0,					0, 0);
	/* a 3 */ glVertex3f(- world_border,			0, 0);
	/* a 4 */ glVertex3f(- world_border,			- world_border, 0);
	/* b 4 */ glVertex3f(0,					- world_border, 0);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0, 0, 1);
	/* c 3 */ glVertex3f(world_width,			0, 0);
	/* c 4 */ glVertex3f(world_width,			- world_border, 0);
	/* d 4 */ glVertex3f(world_width + world_border,	- world_border, 0);
	/* d 3 */ glVertex3f(world_width + world_border,	0, 0);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0, 0, 1);
	/* c 2 */ glVertex3f(world_width,			world_height, 0);
	/* d 2 */ glVertex3f(world_width + world_border,	world_height, 0);
	/* d 1 */ glVertex3f(world_width + world_border,	world_height + world_border, 0);
	/* c 1 */ glVertex3f(world_width,			world_height + world_border, 0);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0, 0, 1);
	/* b 2 */ glVertex3f(0,					world_height, 0);
	/* b 1 */ glVertex3f(0,					world_height + world_border, 0);
	/* a 1 */ glVertex3f(- world_border,			world_height + world_border, 0);
	/* a 2 */ glVertex3f(- world_border,			world_height, 0);
	glEnd();
	glBegin(GL_QUADS);
	glNormal3f(0, 0, 1);
	/* b 2 */ glVertex3f(0,					world_height, 0);
	/* a 2 */ glVertex3f(- world_border,			world_height, 0);
	/* a 3 */ glVertex3f(- world_border,			0, 0);
	/* b 3 */ glVertex3f(0,					0, 0);

	/* b 3 */ glVertex3f(0,					0, 0);
	/* b 4 */ glVertex3f(0,					- world_border, 0);
	/* c 4 */ glVertex3f(world_width,			- world_border, 0);
	/* c 3 */ glVertex3f(world_width,			0, 0);
	
	/* c 3 */ glVertex3f(world_width,			0, 0);
	/* d 3 */ glVertex3f(world_width + world_border,	0, 0);
	/* d 2 */ glVertex3f(world_width + world_border,	world_height, 0);
	/* c 2 */ glVertex3f(world_width,			world_height, 0);
	
	/* c 2 */ glVertex3f(world_width,			world_height, 0);
	/* c 1 */ glVertex3f(world_width,			world_height + world_border, 0);
	/* b 1 */ glVertex3f(0,					world_height + world_border, 0);
	/* b 2 */ glVertex3f(0,					world_height, 0);
	glEnd();


	/*
	gen_material(0.2, 0.6, 0.0);
	glBegin(GL_POLYGON);
	glVertex3f(0, 0, 0);
	glVertex3f(0, 0, world_height);
	glVertex3f(world_width, 0, world_height);
	glVertex3f(world_width, 0, 0);
	glEnd();
	*/

	//glDisable(GL_LIGHTING);
	
	// slunicko
	/*
	*/
	

	// ramecek
	glDisable(GL_LIGHTING);
	glLineWidth(2.0);
	gen_material(0.3, 0.8, 0.1);
	glBegin(GL_LINE_LOOP);
	/* b 3 */ glVertex3f(0,		  0,            0.1);
	/* b 2 */ glVertex3f(0,		  world_height, 0.1);
	/* c 2 */ glVertex3f(world_width, world_height, 0.1);
	/* c 3 */ glVertex3f(world_width, 0,            0.1);
	glEnd();
	glEnable(GL_LIGHTING);

	glEndList();

	DBG_MSG(4, "Done.");


	/* Main */
	glut_master->CallGlutMainLoop();
	return NULL;
}

/*****************************************************************************
 * CallBackResizeFunc -- glut callback
 */
void ScreenGL::CallBackReshapeFunc(int width, int height) {
	DBG_MSG(5, "");
	glViewport(0, 0, width, height);		// viditelna oblast pres cele okno
	glMatrixMode(GL_PROJECTION);			// zacatek modifikace projekcni matice
	
	win_width = width;
	win_height = height;
	glutPostRedisplay();
}

/*****************************************************************************
 * CallBackDisplayFunc -- glut callback
 */
void ScreenGL::CallBackDisplayFunc(void) {
	GLfloat cam_light_pos[4] = { 2.0, 2.0, 2.0, 1.0};
	GLfloat light0_color[4] = { 0.9, 0.9, 0.9 };
	GLfloat light1_color[4] = { 1.0, 1.0, 0.0 };
	GLfloat light_direction[3] = { 0, 0, -1.0 };
	GSymbol *sym = NULL;

	//float sun_pos = step * sun_speed;
	float sun_pos = 80.0;

	DBG_MSG(6, "");
	//GLfloat light_position[] = {-500.0f, -500.0f, 500.0f, 1.0f}; // pozice svetla: jednicka na konci->bodove svetlo

	count_and_update_fps();

	DBG_MSG(7, "sun_pos = %.2f", sun_pos);

	/* update camera coords */
	if (camera_mode != CAM_FREE && actual_msg) {
		actual_msg->get_symbol_rewind();
		while ((sym = actual_msg->get_symbol())) {
			if ((sym->goid == 1 && camera_mode == CAM_KRTEK)
					||(sym->goid == 2 && camera_mode == CAM_ZAHR)) {
				DBG_MSG(5, "Camera follow: %s", (sym->goid) == 1 ? "Krtek":"Zahradnik");
				angle_z = sym->direction;
				angle_x = 15;
				pos_x = sym->x + 0.5                  - 5 * cos(RAD(angle_x)) * sin(RAD(angle_z));
				pos_y = world_height - (sym->y + 0.5) - 5 * cos(RAD(angle_x)) * cos(RAD(angle_z));
				pos_z = 3;
				break;
			}
		}
	}


	/* reset */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(63.4, ((float)win_width)/((float)win_height), 0.1, 10000);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* Vymazani */
	glClearColor(	0.2 + 0.4 * sin(RAD(sun_pos)),
			0.2 + 0.4 * sin(RAD(sun_pos)),
			0.2 + 0.6 * sin(RAD(sun_pos)),
			0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (light_sun)
		glEnable(GL_LIGHT0);
	else
		glDisable(GL_LIGHT0);

	if (light_camera)
		glEnable(GL_LIGHT1);
	else
		glDisable(GL_LIGHT1);
	
	/* Light: Camera */
	glLightfv(GL_LIGHT1, GL_POSITION, cam_light_pos);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_color);
	glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, light_direction);
	glLightf(GL_LIGHT1, GL_SPOT_CUTOFF, 15);
	glLightf(GL_LIGHT1, GL_SPOT_EXPONENT, 0.5f);

	/* srovnani souradnic
	 * +y = vpred
	 * +x = doprava
	 * +z = nahoru
	 */
	glRotatef(-90.0, 1.0, 0.0, 0.0);
	
	/* Camera position & rotation */
	glRotatef(angle_x, 1.0, 0.0, 0.0);
	glRotatef(angle_y, 0.0, 1.0, 0.0);
	glRotatef(angle_z, 0.0, 0.0, 1.0);
	glTranslatef(-pos_x, -pos_y, -pos_z);

	/* Light: Sun */
	light_position[0] = 1000.0 * sin(RAD(sun_pos));
	light_position[1] = 1000.0 * cos(RAD(sun_pos));
	light_position[2] = 1000.0 * sin(RAD(sun_pos));
	light_position[3] =    1.0;
	light0_color[0] = 0.6 + 0.4 * sin(RAD(sun_pos));
	light0_color[1] = 0.5 + 0.5 * sin(RAD(sun_pos));
	light0_color[2] = 0.0 + 1.0 * sin(RAD(sun_pos));
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_color);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	/* Slunicko */
	glPushMatrix();
	glTranslatef(light_position[0], light_position[1], light_position[2]);
	gen_material(light0_color[0], light0_color[1], light0_color[2]);
	glDisable(GL_LIGHTING);
	glutSolidSphere(20, 6, 6);
	glEnable(GL_LIGHTING);
	glPopMatrix();

	/* Render objects */
	glCallList(border_dl);
	
	/* Render world */
	draw_world();

	/* Ok, to je vse */
	glFlush();
	glutSwapBuffers();

	/* jeste prozradime pozici kamery */
	DBG_MSG(4, "pos = [ %0.1f, %0.1f, %0.1f ], angle = [ %0.1f, %0.1f, %0.1f ]",
			pos_x, pos_y, pos_z, angle_x, angle_y, angle_z);
}

/*****************************************************************************
 * CallBackVisibleFunc -- glut callback
 */
void ScreenGL::CallBackVisibleFunc (int vis) {
	DBG_MSG(5, "");
}

/*****************************************************************************
 * CallBackIdleFunc -- glut callback
 */
void ScreenGL::CallBackIdleFunc (void) {
	ScreenMsg *msg = NULL;
	static int count = 0;

	DBG_MSG(6, "");
	/*
	DBG_MSG(6, "pos: [ %lf, %lf, %lf ]", pos_x, pos_y, pos_z);
	pos_z += move_step / 4.0 * cos(RAD(angle_y)) * cos(RAD(angle_x));
	pos_x -= move_step / 4.0 * sin(RAD(angle_y)) * cos(RAD(angle_x));
	pos_y += move_step / 4.0 * sin(RAD(angle_x));
	*/
	if (fifo->pop(&msg) && msg) {

		// World size
		if (msg->have_world_size()) {
			DBG_MSG(5, "World size");
			//msg->get_world_size(&world_width, &world_height);
		}

		// GSymbol
		if (msg->have_symbols()) {
			if(actual_msg)
				actual_msg->unref();

			actual_msg = msg;
			
			DBG_MSG(5, "GSymbol");
			count = (count+1) % 3;
			if (count == 0) {
				glutPostRedisplay();
			}
			step++;
		} else {
			msg->unref();
		}
		
		// done
	}
}
/*****************************************************************************
 * CallBackKeyboardFunc -- glut callback
 * there is no keyboard
 */
void ScreenGL::CallBackKeyboardFunc(unsigned char key, int x, int y) {
	DBG_MSG(5, "key = %c (%d); x = %d; y = %d", key, key, x, y);
	switch (key) {
		
		/* exit */
		case 27:
		case 'q':
		case 'Q':
			kill(getpid(), SIGINT);
			break;

		case 'k':
			glDisable(GL_LIGHTING);
			break;
		case 'l':
			glEnable(GL_LIGHTING);
			break;

		/* Go forward */
		case 'w':
		case 'W':
			pos_z -= move_step * sin(RAD(angle_x));
			pos_x += move_step * cos(RAD(angle_x)) * sin(RAD(angle_z));
			pos_y += move_step * cos(RAD(angle_x)) * cos(RAD(angle_z));
			break;
			
		/* Go backward */
		case 's':
		case 'S':
			pos_z += move_step * sin(RAD(angle_x));
			pos_x -= move_step * cos(RAD(angle_x)) * sin(RAD(angle_z));
			pos_y -= move_step * cos(RAD(angle_x)) * cos(RAD(angle_z));
			break;
		
		/* Go left */
		case 'a':
		case 'A':
			pos_x += move_step * cos(RAD(angle_x)) * sin(RAD(angle_z - 90));
			pos_y += move_step * cos(RAD(angle_x)) * cos(RAD(angle_z - 90));
			break;
		
		/* Go right */
		case 'd':
		case 'D':
			pos_x += move_step * cos(RAD(angle_x)) * sin(RAD(angle_z + 90));
			pos_y += move_step * cos(RAD(angle_x)) * cos(RAD(angle_z + 90));
			break;

		/* Go Up only */
		case '+':
		case '=':
			pos_z += move_step;
			break;

		/* Go Down only */
		case '-':
		case '_':
			pos_z -= move_step;
			break;

		/* camera mode */
		case '1': // Krtek
			camera_mode = CAM_KRTEK;
			break;
		case '2': // Zahradnik
			camera_mode = CAM_ZAHR;
			break;
		case '9': // Free Look
			camera_mode = CAM_FREE;
			break;
		case '0': // Free Look, zero position
			camera_mode = CAM_FREE;
			pos_x   = pos_y   = 0;
			pos_z   = 5;
			angle_x = angle_y = angle_z = 0;
			break;

		/* Lights */
		case '.':
			light_camera = !light_camera;
			break;

		case ',':
			light_sun = !light_sun;
			break;
	}

	if (pos_z <= move_step) {
		pos_z = move_step;
	}

	glutPostRedisplay();
}

/*****************************************************************************
 * CallBackSpecialFunc -- glut callback
 */
void ScreenGL::CallBackSpecialFunc(int key, int x, int y) {
	DBG_MSG(5, "key = %d; x = %d; y = %d", key, x, y);
	switch (key) {
		case GLUT_KEY_LEFT:
			angle_z -= angle_step;
			break;
		case GLUT_KEY_RIGHT:
			angle_z += angle_step;
			break;
		case GLUT_KEY_UP:
			angle_x -= angle_step;
			break;
		case GLUT_KEY_DOWN:
			angle_x += angle_step;
			break;
	}
	DBG_MSG(4, "angle_x = %f; angle_y = %f", angle_x, angle_y);
	glutPostRedisplay();
}

void ScreenGL::CallBackMotionFunc(int x, int y) {
	DBG_MSG(5, "x = %d; y = %d", x, y);
	if (mouse_move) {
		angle_z += (x - mouse_last_z);
		angle_x += (y - mouse_last_y);
		mouse_last_z = x;
		mouse_last_y = y;
		glutPostRedisplay();
	}
}

void ScreenGL::CallBackMouseFunc(int button, int state, int x, int y) {
	DBG_MSG(5, "btn = %d, state = %d, x = %d; y = %d", button, state, x, y);
	mouse_move = (state == 0);
	mouse_last_z = x;
	mouse_last_y = y;
}

/*****************************************************************************
 * gen_material
 * nastaveni nejakeho rozumneho materialu dane barvy
 */
void ScreenGL::gen_material(GLfloat r, GLfloat g, GLfloat b) {
	const float ambient = 0.8;
	//const float specular = 3;
	
	GLfloat materialAmbient[]={r*ambient, g*ambient, b*ambient, 1.0f};
	GLfloat materialDiffuse[]={r, g, b, 1.0f};
	/*
	GLfloat materialSpecular[]={r*specular, g*specular, b*specular, 1.0f};
	GLfloat materialShininess[]={150.0f};
	// */
	
	glColor3f(r, g, b);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, materialAmbient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, materialDiffuse);
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	/*
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, materialSpecular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, materialShininess);
	// */
}

/*****************************************************************************
 * count_and_update_fps
 * pocitadlo snimku za sekundu, pise hodnotu do titulku
 */
void ScreenGL::count_and_update_fps(void) {
	/* count FPS */
	int now = time(NULL);	// TODO neco presnejsiho
	if (now > fps_old_time + fps_update_rate || fps_old_time == 0) {
		float fps = fps_counter / (now - fps_old_time);
		fps_old_time = now;
		fps_counter = 0;
		
		DBG_MSG(4, "fps = %0.3f", fps);

		char str[256];
		sprintf(str, title, (float)fps);
		glutSetWindowTitle(str);
	}
	fps_counter += 1;
}


/*****************************************************************************
 * kresleni objektu herniho sveta
 */
void ScreenGL::draw_world(void) {
	static const float color[16][3] = {
		/* R    G    B */
		{ 0.0, 0.0, 0.0 },
		{ 0.0, 0.0, 0.5 },
		{ 0.0, 0.5, 0.0 },
		{ 0.5, 0.0, 0.0 },
		{ 0.0, 0.4, 0.4 },
		{ 0.4, 0.0, 0.4 },
		{ 0.4, 0.4, 0.0 },
		{ 0.4, 0.4, 0.4 },
		
		{ 0.2, 0.2, 0.2 },
		{ 0.0, 0.0, 0.8 },
		{ 0.0, 0.8, 0.0 },
		{ 0.8, 0.0, 0.0 },
		{ 0.0, 0.8, 0.8 },
		{ 0.8, 0.0, 0.8 },
		{ 0.8, 0.8, 0.0 },
		{ 0.8, 0.8, 0.8 },
	};
	GSymbol *sym = NULL;

	if (!actual_msg) {
		DBG_MSG(5, "no message");
		return;
	}

	glLineWidth(2);
	DBG_MSG(6, "");
	actual_msg->get_symbol_rewind();
	while ((sym = actual_msg->get_symbol())) {
		DBG_MSG(8, "goid = %d", sym->goid);
		 
		// move to object position
		glPushMatrix();
		glTranslatef(sym->x + 0.5, world_height - (sym->y + 0.5), 0.0);
		glRotatef(-sym->direction, 0.0, 0.0, 1.0);
		
		// draw object
		gen_material(color[sym->color % 16][0], color[sym->color % 16][1], color[sym->color % 16][2]);
		if (sym->z == 0) {
			glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glVertex3f(-0.5, -0.5, 0.0);
			glVertex3f(+0.5, -0.5, 0.0);
			glVertex3f(+0.5, +0.5, 0.0);
			glVertex3f(-0.5, +0.5, 0.0);
			glEnd();
		} else {
			glDisable(GL_LIGHTING);
			glBegin(GL_LINES);
			glVertex3f(0.0, 0.0, 0.2);
			glVertex3f(0.0, 1.5, 0.2);
			glEnd();
			glEnable(GL_LIGHTING);
			glutSolidCube(1.0);
		}
		glPopMatrix();
	}
	DBG_MSG(6, "Done");
}

