/*
 * screen_window.h -- OpenGL screen window
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#ifndef JK_SCREEN_GL_H
#define JK_SCREEN_GL_H

#include <time.h>

#include "thread.h"
#include "glut_window.h"
#include "glut_master.h"
#include "screen_msg.h"

class ScreenGL : public GlutWindow, public Thread {
	public:
		ScreenGL(int *argc, char *argv[], ScreenFifo *fifo);
		~ScreenGL(void);

	private:
		void *main(void);

		ScreenFifo *fifo;
		ScreenMsg *actual_msg;
		
		static GlutMaster *glut_master;
		char *title;	// window title

		/* fps counter */
		time_t fps_old_time;		// last fps view
		float fps_counter;		// temporary value
		float fps_update_rate;		// seconds between fps updates

		/* rotation */
		int win_width, win_height;
		int world_width, world_height;
		int world_border;
		int step;
		float sun_speed;
		float angle_x, angle_y, angle_z, pos_x, pos_y, pos_z;
		float move_step, angle_step;
		float light_position[4];
		enum { CAM_KRTEK, CAM_ZAHR, CAM_FREE } camera_mode;

		/* mouse movement */
		int mouse_move;
		int mouse_last_z, mouse_last_y;

		/* lights */
		bool light_sun;
		bool light_camera;

		void CallBackReshapeFunc (int width, int height);
		void CallBackDisplayFunc (void);
		void CallBackVisibleFunc (int vis);
		void CallBackIdleFunc (void);
		void CallBackKeyboardFunc (unsigned char key, int x, int y);
		void CallBackSpecialFunc(int key, int x, int y);
		void CallBackMotionFunc(int x, int y);
		void CallBackMouseFunc(int button, int state, int x, int y);
		
		void count_and_update_fps(void);
		void gen_material(GLfloat r, GLfloat g, GLfloat b);

		void draw_world(void);
		
		int border_dl;
};

#endif

