/*
 * screen_msg.cc
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include "screen_msg.h"

#define CLASS "ScreenMsg"
#define DEBUG 0
#include "debug.h"


ScreenMsg::ScreenMsg(void) {
	DBG_MSG(5, "");
	sample_msg = NULL;
	rule = NULL;
	width = -1;
	height = -1;
	is_status_set = false;
}

ScreenMsg::~ScreenMsg(void) {
	DBG_MSG(5, "");
	if (rule)
		rule->unref();
	if (sample_msg)
		sample_msg->unref();
	for (GSymbolList::iterator i = symbol.begin(); i != symbol.end(); i++) {
		if (*i)
			delete *i;
	}
}

bool ScreenMsg::have_symbols(void) {
	DBG_MSG(7, "");
	return !symbol.empty();
}

bool ScreenMsg::have_sample_msg(void) {
	DBG_MSG(7, "");
	return sample_msg != NULL;
}

bool ScreenMsg::have_freq_rule(void) {
	DBG_MSG(7, "");
	return rule && !rule->empty();
}

bool ScreenMsg::have_world_size(void) {
	DBG_MSG(7, "");
	return width >= 0 && height >= 0;
}

bool ScreenMsg::have_status(void) {
	DBG_MSG(7, "");
	return is_status_set;
}

void ScreenMsg::set_world_size(int w, int h) {
	DBG_MSG(7, "");
	width = w;
	height = h;
}

void ScreenMsg::get_world_size(int *w, int *h) {
	DBG_MSG(7, "");
	*w = width;
	*h = height;
}

void ScreenMsg::add_symbol(GSymbol *sym) {
	DBG_MSG(8, "");
	if(sym)
		symbol.push_back(sym);
}

void ScreenMsg::get_symbol_rewind(void) {
	DBG_MSG(8, "");
	symbol_i = symbol.begin();
}


GSymbol *ScreenMsg::get_symbol(void) {
	DBG_MSG(8, "");
	return symbol_i != symbol.end() ? *(symbol_i++): NULL;
}

void ScreenMsg::add_sample_msg(SampleMsg *msg) {
	DBG_MSG(7, "");
	sample_msg = msg;
	sample_msg->ref();
}

SampleMsg *ScreenMsg::get_sample_msg(void) {
	DBG_MSG(7, "");
	return sample_msg;
}

void ScreenMsg::add_freq_rule(ScreenFreqRule *rule) {
	DBG_MSG(7, "");
	this->rule = rule;
	this->rule->ref();
}

ScreenFreqRule *ScreenMsg::get_freq_rule(void) {
	DBG_MSG(7, "");
	return rule;
}

void ScreenMsg::set_status(int k_key, int k_score, int z_key, int z_score) {
	this->is_status_set = true;
	this->k_key = k_key;
	this->k_score = k_score;
	this->z_key = z_key;
	this->z_score = z_score;
}
void ScreenMsg::get_status(int *k_key, int *k_score, int *z_key, int *z_score) {
	*k_key = this->k_key;
	*k_score = this->k_score;
	*z_key = this->z_key;
	*z_score = this->z_score;
}

#undef CLASS

