/*
 * screen_msg.h
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#ifndef JK_SCREEN_MSG_H
#define JK_SCREEN_MSG_H

#include <list>

#include "sample_msg.h"
#include "game_object.h"
#include "screen.h"
#include "ref.h"
#include "fifo.h"

/*
 * pozadavky:
 *	- nezavislost na stavu hry => musi se poslat vse co bude kresleno
 *
 */
class ScreenMsg : public Ref {
	public:
		ScreenMsg(void);
		~ScreenMsg(void);
		
		bool have_symbols(void);
		bool have_sample_msg(void);
		bool have_freq_rule(void);
		bool have_world_size(void);
		bool have_status(void);

		void get_world_size(int *w, int *h);
		void set_world_size(int w, int h);

		void add_symbol(GSymbol *sym);
		GSymbol *get_symbol(void);
		void get_symbol_rewind(void);
		
		void add_sample_msg(SampleMsg *msg);
		SampleMsg *get_sample_msg(void);

		void add_freq_rule(ScreenFreqRule *rule);
		ScreenFreqRule *get_freq_rule(void);

		void set_status(int k_key, int k_score, int z_key, int z_score);
		void get_status(int *k_key, int *k_score, int *z_key, int *z_score);

	private:
		typedef std::list<GSymbol *> GSymbolList;

		int width, height;
		GSymbolList symbol;		// seznam symbolu k vykresleni
		GSymbolList::iterator symbol_i;
		
		SampleMsg *sample_msg;		// frekvencni spektrum a par takovych veci
		ScreenFreqRule *rule;		// pravitko s frekvencema
		
		bool is_status_set;
		int k_key, k_score, z_key, z_score;
};

typedef Fifo<ScreenMsg *> ScreenFifo;

#endif

